/*
//    Paradigm PDREMOTE/ROM Target System Interface
//
//    UniROM URCOM Driver -   Release 2.0.
//
//    Copyright (C) 1989, 1991 Paradigm Systems.  All rights reserved.
//    Portions Copyright (C) 1988 Borland International.  All rights reserved.
//    Portions Copyright (C) 1996 TechTools. All rights reserved.
//
//    This module provides support for the UniROM in POLLED MODE
//    or INTERRUPT mode. If COMMINT is defined to be the NMI interrupt
//    vector (2), an interface using the NMI input for receive interrupts can
//    be built.  If COMMINT is left undefined, the driver defaults to polled
//    mode.
//
//    If INTERRUPT MODE is selected, a PC Compatible
//    8259 Interrupt controller is assumed. If your Target differs from
//    this configuration, replace the "8259 Interrupt Controller Defines"
//    area with the appropriate defines for your Target.
//
//    It is impossible for us to anticipate all of the various possible
//    hardware combinations of processor, interrupt controller etc.
//
//    It is important that you take the time to check to ensure
//    that the register addresses and values defined in this file
//    and the various header files used are appropriate for your
//    particular hardware configuration.
//
//    Functions requiring customization are
//       COMMinit          Interrupt Hardware initialization
//       COMMrcvint        Interrupt controller clean-up
//
//    The only customization required involves the initialization of
//    the interrupt controller and any cleanup required after an
//    interrupt.  Look carefully at the end of COMMinit and the beginning
//    of COMMrcvint.  A polled mode driver should not require modification
//    (with the exception of specifying the URCOM SEGMENT below).
*/

#include <dos.h>                    /* Enable/disable functions */
#include "typedefs.h"               /* Common definitions and prototypes */
#include "target.h"                 /* Target system specific information */
#include "helpers.h"                /* PDREMOTE/ROM helper functions */


/////////////////////////////////////////////////////////////////
//                                                             //
//          DEFINES for UniROM                                 //
//                                                             //

    // base SEGMENT for UniROM's URCOM board
#define URCOM_BASE  0xfffc
    
    // Address scale factor: 1 for 8 bit EPROM data paths
    //                       2 for 16 bit EPROM data paths
    //                       4 for 32 bit EPROM data paths  
#define SCALE 1

//                                                             //
//                                                             //
/////////////////////////////////////////////////////////////////


#if   defined(COMMINT)
#if   COMMINT != 2
#define  IOB8259     0x0020         /* PC-compatible 8259 interrupt controller */
#define SC  1                       // I/O port scale factor for 8259
#include "i8259.h"                  /* 8259A interrupt controller definitions */
#define  INTBASE     8              /* Hardware interrupt for bus interrupt 0 */
#define  INTNUM      3              /* IBM PC COM 2 interrupt number */

#else    /* !NMI */
#define  INTBASE     0              /* Base interrupt */
#define  INTNUM      2              /* NMI interrupt vector */
#endif   /* !NMI */

#if   (COMMINT != INTBASE + INTNUM)
#error   Incorrect COMM interrupt vector defined in TARGET.H
#endif   /* COMMINT check */
#endif   /* COMMINT */


// global URCOM pointers
static BYTE_PTR volatile tx_base;
static BYTE_PTR volatile rx_buff;
static BYTE_PTR volatile status;


/*
//    Function:  COMMsendchar
//
//    Send a character as soon as the transmitter is ready to accept it.
*/

void     COMMsendchar(BYTE bTXchar)
{unsigned char dummy;

 // wait for TX buffer to empty
 while((*status & 0x01))
    continue;
 
 // send data
 dummy = *(tx_base + SCALE*(bTXchar & 0x07));       // encode d0-d2
 bTXchar = bTXchar >> 3;
 dummy = *(tx_base + SCALE*(bTXchar & 0x07));       // encode d3-d5
 bTXchar = bTXchar >> 3;
 dummy = *(tx_base + SCALE*((bTXchar & 0x03)+8));    // encode d6-d7 
                                                     // and set FULL FLAG
}



/*
//    Function:  COMMrcvint
//
//    This function serves as the receive interrupt routine for an
//    interrupt-mode kernel.  It also serves as a receive POLLING
//    routine for a POLLED-mode kernel.
//
//
//    WARNING: No local (class auto) data can be allocated in the interrupt
//             handler since the stack is cleaned up elsewhere if a Ctrl-Break
//             interrupt is processed.  Note that both p and wCount are
//             declared as static and do not occupy space on the stack.
*/

void  interrupt far  COMMrcvint(void)
{
 static BYTE    *p ;              /* Pointer to the packet */
 static WORD    wCount ;          /* Loop counter */

#if !defined(COMMINT)
 disable();
#endif

 if(*status & 0x02)      // IF a receive interrupt or poll
  {if(!wCount)           //   IF first character
    {p = pCOMMpacket;    //     Initialize the pointer into the packet
     wCount = *rx_buff;  //     get packet length
     *p++ = wCount;      //     stuff length into the buffer
     if (wCount > 0)     //     IF NOT a NULL packet -
      COMMsendchar(0) ;  //       send back a NULL  (ACK)
     else
      bGotPacket = 1;    //     ELSE NULL packet complete
    }
   else                  //   ELSE middle/end of packet
    {*p++ = *rx_buff;    //     retrieve and buffer character
     wCount--;           //     one less character to receive
     if(!wCount)         //     IF last character of packet
      bGotPacket = 1;    //       packet complete
    }
  }

#if (defined(COMMINT) && (COMMINT != 2))
   HLPRoutportb(OCW2, EOI) ;  //Issue the EOI to interrupt controller
#endif

 if(bGotPacket)
   Progtestpacket();

}



/*
//    Function:  COMMgotpacket
//
//    See if a received packet is waiting.  Returns 1 if a packet is
//    available, returns 0 for no packet.  Note that this function returns
//    1 once, the next call will return 0.
//
//    Note that this routine defines a critical region which must not be
//    violated.
*/

WORD COMMgotpacket(void)
{ BYTE bStatus ;

#if   !defined(COMMINT)  // IF POLL MODE
  if(*status & 0x02)     //   AND IF first character waiting
    while(!bGotPacket)   //     UNTIL all characters received
      COMMrcvint() ;     //       poll for characters
#endif

  /* Get the current status, then immediately clear the flag if set */
  if ((bStatus = bGotPacket) == 1)
    bGotPacket = 0 ;

  return   (WORD) bStatus ;
}



/*
//    Function:  COMMinit
//
//    Initialize the communications port, install the receive
//    interrupt handler and initialize statics.
*/

void     COMMinit(void)
{ unsigned char dummy;
  disable() ;

  tx_base = (((BYTE_PTR) MK_FP(URCOM_BASE,0))) ;
  rx_buff = (((BYTE_PTR) MK_FP(URCOM_BASE,0x0c*SCALE))) ;
  status  = (((BYTE_PTR) MK_FP(URCOM_BASE,0x0d*SCALE))) ;
  
  while ((*status & 0x03) != 0x00) // wait for URCOM to initilize
    dummy = *rx_buff;              //   absorb any RX chars while waiting

  // if interrupt mode - Initialize and enable interrupts
#if   defined(COMMINT)
   /* Load our interrupt routine address */
  HLPRsetvect((INTBASE + INTNUM), (void far *) COMMrcvint) ;

#if   COMMINT != 2

//
// LOOK carefully at the following code to insure that it is
// correct for YOUR particular interrupt controller structure.
//
   /* Enable the selected interrupt level */
  HLPRoutportb(IMR, HLPRinportb(IMR) & ~(1 << INTNUM)) ;

#endif   /* !NMI */
#endif   /* COMMINT */
  enable() ;

}
